@@ -6,7 +6,7 @@ use Sub::Data::Recursive;
 use POSIX qw/strftime/;
 use Getopt::Long qw/GetOptionsFromArray/;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 my $MAX_DEPTH = 10;
 
@@ -24,6 +24,8 @@ my $MAYBE_UNIXTIME = join '|', (
 
 my $UNIXTIMESTAMP_KEY = '';
 
+my $GMTIME;
+
 sub new {
     my $class = shift;
     my @argv  = @_;
@@ -64,16 +66,38 @@ sub process {
         return $line;
     }
     else {
-        Sub::Data::Recursive->invoke(\&_split_lf => $decoded) if $self->opt('x');
-        $self->{_depth} = $self->opt('depth');
-        $self->_recursive_decode_json($decoded);
-        Sub::Data::Recursive->invoke(\&_split_comma => $decoded) if $self->opt('xx');
-        Sub::Data::Recursive->invoke(\&_split_label => $decoded) if $self->opt('xxx');
-        Sub::Data::Recursive->massive_invoke(\&_convert_timestamp => $decoded) if $self->opt('xxxx');
-        return $self->{_json}->encode($decoded);
+        $self->_recursive_process($decoded);
+        return $self->{_json}->encode($decoded)
     }
 }
 
+sub _recursive_process {
+    my ($self, $decoded) = @_;
+
+    $self->_recursive_pre_process($decoded);
+
+    $self->{_depth} = $self->opt('depth');
+    $self->_recursive_decode_json($decoded);
+
+    $self->_recursive_post_process($decoded);
+}
+
+sub _recursive_pre_process {
+    my ($self, $decoded) = @_;
+
+    Sub::Data::Recursive->invoke(\&_split_lf => $decoded) if $self->opt('x');
+}
+
+sub _recursive_post_process {
+    my ($self, $decoded) = @_;
+
+    Sub::Data::Recursive->invoke(\&_split_lf => $decoded) if $self->opt('x');
+    Sub::Data::Recursive->invoke(\&_split_comma => $decoded) if $self->opt('xx');
+    Sub::Data::Recursive->invoke(\&_split_label => $decoded) if $self->opt('xxx');
+    Sub::Data::Recursive->massive_invoke(\&_convert_timestamp => $decoded) if $self->opt('xxxx') || $self->opt('timestamp_key');
+    Sub::Data::Recursive->invoke(\&_trim => $decoded);
+}
+
 sub _split_lf {
     my $line = $_[0];
 
@@ -89,7 +113,7 @@ sub _split_comma {
 
     chomp $line;
 
-    return $line if length $line < 128 || $line !~ m!, ! || $line =~ m!\\!;
+    return $line if $line !~ m!, ! || $line =~ m!\\!;
 
     my @elements = split /,\s+/, $line;
 
@@ -101,7 +125,7 @@ sub _split_label {
 
     chomp $line;
 
-    return $line if length $line < 128 || $line =~ m!\\!;
+    return $line if $line =~ m!\\!;
 
     $line =~ s!([])>])\s+([[(<])!$1$2!g;
     $line =~ s!((\[[^])>]+\]|\([^])>]+\)|<[^])>]+>))!$1\n!g; # '\n' already replaced by --x option
@@ -123,13 +147,33 @@ sub _convert_timestamp {
             || ($LAST_VALUE =~ m!(?:$MAYBE_UNIXTIME)!i && $line =~ m!(\d+(\.\d+)?)!)
     ) {
         if (my $date = _ts2date($1, $2)) {
-            $_[0] = "$date = $line";
+            $_[0] = $date;
         }
     }
 
     $LAST_VALUE = $line;
 }
 
+sub _trim {
+    my $line = $_[0];
+
+    my $trim = 0;
+
+    if ($line =~ m!^[\s\t\r\n]+!) {
+        $line =~ s!^[\s\t\r\n]+!!;
+        $trim = 1;
+    }
+
+    if ($line =~ m![\s\t\r\n]+$!) {
+        $line =~ s![\s\t\r\n]+$!!;
+        $trim = 1;
+    }
+
+    if ($trim) {
+        $_[0] = $line;
+    }
+}
+
 sub _ts2date {
     my $unix_timestamp = shift;
     my $msec           = shift || '';
@@ -141,7 +185,8 @@ sub _ts2date {
             $msec = ".$msec";
             $unix_timestamp = int($unix_timestamp / 1000);
         }
-        return strftime('%Y-%m-%d %H:%M:%S', localtime($unix_timestamp)) . $msec;
+        my @t = $GMTIME ? gmtime($unix_timestamp) : localtime($unix_timestamp);
+        return strftime('%Y-%m-%d %H:%M:%S', @t) . $msec;
     }
 }
 
@@ -178,6 +223,7 @@ sub _parse_opt {
         'xxx'       => \$opt->{xxx},
         'xxxx'      => \$opt->{xxxx},
         'timestamp-key=s' => \$opt->{timestamp_key},
+        'gmtime'    => \$opt->{gmtime},
         'h|help'    => sub {
             $class->_show_usage(1);
         },
@@ -195,6 +241,8 @@ sub _parse_opt {
 
     $UNIXTIMESTAMP_KEY = $opt->{timestamp_key};
 
+    $GMTIME = $opt->{gmtime};
+
     return $opt;
 }
 
@@ -213,7 +261,7 @@ __END__
 
 =head1 NAME
 
-App::jl - Recursive JSON decoder
+App::jl - Show JSON Log Nicely
 
 
 =head1 SYNOPSIS
@@ -251,7 +299,7 @@ The parser of the line
 
 =begin html
 
-<a href="http://travis-ci.org/bayashi/App-jl"><img src="https://secure.travis-ci.org/bayashi/App-jl.png?_t=1560229824"/></a> <a href="https://coveralls.io/r/bayashi/App-jl"><img src="https://coveralls.io/repos/bayashi/App-jl/badge.png?_t=1560229824&branch=master"/></a>
+<a href="http://travis-ci.org/bayashi/App-jl"><img src="https://secure.travis-ci.org/bayashi/App-jl.png?_t=1560527336"/></a> <a href="https://coveralls.io/r/bayashi/App-jl"><img src="https://coveralls.io/repos/bayashi/App-jl/badge.png?_t=1560527336&branch=master"/></a>
 
 =end html
 
